/**
 * OW2 Util
 * Copyright (C) 2008
 * Contact: easybeans@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: IFXmlHelperImpl.java
* --------------------------------------------------------------------------
*/

package org.ow2.util.plan.deploy.deployable.internal.xmlhelper;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class XmlHelper {

    /**
     * The parser that will be used for validation with namespaces.
     */
    private SAXParser namespaceAwareParser = null;

    /**
     * The parser that will be used for parsing when namespaces are not
     * considered.
     */
    private SAXParser namespaceUnawareParser = null;

    /**
     * Default constructor for XmlHelperImpl.
     *
     * @throws ParserConfigurationException in case of SAX misconfiguration.
     * @throws SAXException is SAX errors occured.
     */
    public XmlHelper() throws ParserConfigurationException, SAXException {
        SAXParserFactory factory = SAXParserFactory.newInstance();;
        namespaceUnawareParser = factory.newSAXParser();
        factory.setNamespaceAware(true);
        namespaceAwareParser = factory.newSAXParser();
    }

    /**
     * {@inheritDoc}
     */
    public synchronized boolean isWellFormedXml(final File file) throws IllegalArgumentException, IOException {
        try {
            namespaceUnawareParser.parse(file, new DefaultHandler());
            return true;
        } catch (SAXException e) {
            return false;
        }
    }

    /**
     * {@inheritDoc}
     */
    public synchronized boolean isLikelyXml(final File file) throws IllegalArgumentException, IOException {
        try {
            namespaceUnawareParser.parse(file, new CheckFirstElementHandler());
            return false;
        } catch (GetOutException e) {
            return true;
        } catch (SAXException e) {
            return false;
        }

    }

    /**
     * {@inheritDoc}
     */
    public synchronized String getRootElementNameSpace(final File file) throws IOException, SAXException {
        try {
            namespaceAwareParser.parse(file, new CheckFirstElementHandler());
            return null; // else the compiler is unhappy
        } catch (GetOutException e) {
            return e.getNameSpace();
        }
    }

    /**
     * Exception used to cancel the parsing once the first element
     * of an XML file has been read.
     * @author mleduque
     */
    static class GetOutException extends SAXException {

        private static final long serialVersionUID = 27568468415776L;

        /**
         * The namespace of the first element of the file.
         */
        private String nameSpace = null;

        /**
         * the first element name.
         */
        private String elementName = null;

        /**
         *
         * @param nameSpace the name space.
         * @param elementName the element name.
         */
        public GetOutException(final String nameSpace, final String localName) {
            this.nameSpace = nameSpace;
            this.elementName = localName;
        }

        /**
         * Sets the namespace.
         * @param nameSpace the new namespace.
         */
        public void setNameSpace(final String nameSpace) {
            this.nameSpace = nameSpace;
        }

        /**
         * Gets the namespace.
         * @return the namespace.
         */
        public String getNameSpace() {
            return nameSpace;
        }

        /**
         * Sets the element name.
         * @param elementName the new element name.
         */
        public void setElementName(final String localName) {
            this.elementName = localName;
        }

        /**
         * Gets the element name.
         * @return the element name.
         */
        public String getElementName() {
            return this.elementName;
        }
    }

    /**
     * SAX handler for the xml helper.
     * @author mleduque
     */
    static class CheckFirstElementHandler extends DefaultHandler {

        /**
         * {@inheritDoc}
         */
        @Override
        public void startElement(final String uri, final String localName, final String qName,
                final org.xml.sax.Attributes attributes) throws SAXException {
                /* We've found the root element
                 * Abort the parsing. */
                throw new GetOutException(uri, localName);
        }

    }
}
